@@ -10,10 +10,6 @@
 
 #include "ScintillaQt.h"
 #include "PlatQt.h"
-#ifdef SCI_LEXER
-#include "LexerModule.h"
-#include "ExternalLexer.h"
-#endif
 
 #include <QApplication>
 #include <QDrag>
@@ -41,7 +37,7 @@ ScintillaQt::ScintillaQt(QAbstractScrollArea *parent)
 	// On OS X drawing text into a pixmap moves it around 1 pixel to
 	// the right compared to drawing it directly onto a window.
 	// Buffered drawing turned off by default to avoid this.
-	WndProc(SCI_SETBUFFEREDDRAW, false, 0);
+	view.bufferedDraw = false;
 
 	Init();
 
@@ -65,6 +61,8 @@ void ScintillaQt::execCommand(QAction *action)
 #if defined(Q_OS_WIN)
 static const QString sMSDEVColumnSelect("MSDEVColumnSelect");
 static const QString sWrappedMSDEVColumnSelect("application/x-qt-windows-mime;value=\"MSDEVColumnSelect\"");
+static const QString sVSEditorLineCutCopy("VisualStudioEditorOperationsLineCutCopyClipboardTag");
+static const QString sWrappedVSEditorLineCutCopy("application/x-qt-windows-mime;value=\"VisualStudioEditorOperationsLineCutCopyClipboardTag\"");
 #elif defined(Q_OS_MAC)
 static const QString sScintillaRecPboardType("com.scintilla.utf16-plain-text.rectangular");
 static const QString sScintillaRecMimeType("text/x-scintilla.utf16-plain-text.rectangular");
@@ -193,6 +191,14 @@ static void AddRectangularToMime(QMimeData *mimeData, QString su)
 #endif
 }
 
+static void AddLineCutCopyToMime(QMimeData *mimeData)
+{
+#if defined(Q_OS_WIN)
+	// Add an empty marker
+	mimeData->setData(sVSEditorLineCutCopy, QByteArray());
+#endif
+}
+
 static bool IsRectangularInMime(const QMimeData *mimeData)
 {
 	QStringList formats = mimeData->formats();
@@ -217,6 +223,23 @@ static bool IsRectangularInMime(const QMimeData *mimeData)
 	return false;
 }
 
+static bool IsLineCutCopyInMime(const QMimeData *mimeData)
+{
+	QStringList formats = mimeData->formats();
+	for (int i = 0; i < formats.size(); ++i) {
+#if defined(Q_OS_WIN)
+		// Visual Studio Line Cut/Copy markers
+		// If line cut/copy made by this application, see base name.
+		if (formats[i] == sVSEditorLineCutCopy)
+			return true;
+		// Otherwise see wrapped name.
+		if (formats[i] == sWrappedVSEditorLineCutCopy)
+			return true;
+#endif
+	}
+	return false;
+}
+
 bool ScintillaQt::ValidCodePage(int codePage) const
 {
 	return codePage == 0
@@ -299,7 +322,6 @@ void ScintillaQt::ReconfigureScrollBars()
 void ScintillaQt::CopyToModeClipboard(const SelectionText &selectedText, QClipboard::Mode clipboardMode_)
 {
 	QClipboard *clipboard = QApplication::clipboard();
-	clipboard->clear(clipboardMode_);
 	QString su = StringFromSelectedText(selectedText);
 	QMimeData *mimeData = new QMimeData();
 	mimeData->setText(su);
@@ -307,6 +329,10 @@ void ScintillaQt::CopyToModeClipboard(const SelectionText &selectedText, QClipbo
 		AddRectangularToMime(mimeData, su);
 	}
 
+	if (selectedText.lineCopy) {
+		AddLineCutCopyToMime(mimeData);
+	}
+
 	// Allow client code to add additional data (e.g rich text).
 	emit aboutToCopy(mimeData);
 
@@ -332,6 +358,7 @@ void ScintillaQt::PasteFromMode(QClipboard::Mode clipboardMode_)
 	QClipboard *clipboard = QApplication::clipboard();
 	const QMimeData *mimeData = clipboard->mimeData(clipboardMode_);
 	bool isRectangular = IsRectangularInMime(mimeData);
+	bool isLine = SelectionEmpty() && IsLineCutCopyInMime(mimeData);
 	QString text = clipboard->text(clipboardMode_);
 	QByteArray utext = BytesForDocument(text);
 	std::string dest(utext.constData(), utext.length());
@@ -341,7 +368,7 @@ void ScintillaQt::PasteFromMode(QClipboard::Mode clipboardMode_)
 	UndoGroup ug(pdoc);
 	ClearSelection(multiPasteMode == SC_MULTIPASTE_EACH);
 	InsertPasteShape(selText.Data(), selText.Length(),
-		selText.rectangular ? pasteRectangular : pasteStream);
+		isRectangular ? pasteRectangular : (isLine ? pasteLine : pasteStream));
 	EnsureCaretVisible();
 }
 
@@ -459,9 +486,9 @@ bool ScintillaQt::ChangeIdle(bool on)
 			idler.state = false;
 			qIdle = static_cast<QTimer *>(idler.idlerID);
 			qIdle->stop();
-			disconnect(qIdle, SIGNAL(timeout()), 0, 0);
+			disconnect(qIdle, SIGNAL(timeout()), nullptr, nullptr);
 			delete qIdle;
-			idler.idlerID = 0;
+			idler.idlerID = {};
 		}
 	}
 	return true;
@@ -545,12 +572,14 @@ CaseFolder *ScintillaQt::CaseFolderForEncoding()
 				// Only for single byte encodings
 				for (int i=0x80; i<0x100; i++) {
 					char sCharacter[2] = "A";
-					sCharacter[0] = i;
+					sCharacter[0] = static_cast<char>(i);
 					QString su = codec->toUnicode(sCharacter, 1);
 					QString suFolded = su.toCaseFolded();
-					QByteArray bytesFolded = codec->fromUnicode(suFolded);
-					if (bytesFolded.length() == 1) {
-						pcf->SetTranslation(sCharacter[0], bytesFolded[0]);
+					if (codec->canEncode(suFolded)) {
+						QByteArray bytesFolded = codec->fromUnicode(suFolded);
+						if (bytesFolded.length() == 1) {
+							pcf->SetTranslation(sCharacter[0], bytesFolded[0]);
+						}
 					}
 				}
 				return pcf;
@@ -558,7 +587,7 @@ CaseFolder *ScintillaQt::CaseFolderForEncoding()
 				return new CaseFolderDBCS(QTextCodec::codecForName(charSetBuffer));
 			}
 		}
-		return 0;
+		return nullptr;
 	}
 }
 
@@ -704,12 +733,6 @@ sptr_t ScintillaQt::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam)
 		case SCI_GETDIRECTPOINTER:
 			return reinterpret_cast<sptr_t>(this);
 
-#ifdef SCI_LEXER
-		case SCI_LOADLEXERLIBRARY:
-			LexerManager::GetInstance()->Load(reinterpret_cast<const char *>(lParam));
-			break;
-#endif
-
 		default:
 			return ScintillaBase::WndProc(iMessage, wParam, lParam);
 		}
